
#include "cvmultiwin.h"

int		CvMultiWin::M_SPC				= 10;
CvFont*	CvMultiWin::M_FONT				= NULL;
bool	CvMultiWin::M_CLASS_INITIALIZED	= false;

// The user is required to make an initialize() call before using
// the multi-window.  This call should indicate the name for the
// window (as per cvNamedWindow()), as well as the width and height
// of the panel images.  c,r specify the number of columns and
// rows (respectively) of panels to create.  depth and channels
// are passed directly to the multi-panel image.
//
void CvMultiWin::initialize( 
	const char* name, 
	int w, 
	int h, 
	int Nc, 
	int Nr, 
	int depth, 
	int channels,
	int flags
) {
	assert( m_initialized==false );

	// A Font object exists which the CvMultiWin class uses to 
	// write labels (when they are present) on the subwindows.
	//  This Font is a static member of the class, so is only
	// initialized when the class has never been initialized.
	// (This will happen on the first call to CvMultiWin::initialize()
	// obviously.)
	//
	if( ! M_CLASS_INITIALIZED ) {
		M_FONT = new CvFont;
//M_FONT = (CvFont*) malloc( sizeof( CvFont ) );
		assert( M_FONT );
		cvInitFont( M_FONT, CV_FONT_VECTOR0, (float)0.5, (float)0.5, 0.0, 2 );
		M_CLASS_INITIALIZED = true;
	}

	cvNamedWindow( name, flags );
	strncpy( m_szWindowName, name, 256 );

	m_width				= (w+M_SPC)*Nc + M_SPC;
	m_height			= (h+M_SPC)*Nr + M_SPC;
	m_panel_width		= w;
	m_panel_height		= h;
	m_columns			= Nc;
	m_rows				= Nr;

	m_implicit_target	= 0;

	if( m_image != NULL ) { cvReleaseImage( &m_image ); }
	m_image = cvCreateImage( cvSize( m_width, m_height ), depth, channels );
	cvAddS( m_image, CV_RGB(128,128,128), m_image );
//	cvAddS( m_image, cvScalar(128,128,128,0), m_image );

	// Every multiwin has a set of
	// labels, one for each panel.  Whenever a panel is drawn, the label
	// will automatically be labeled with whatever label exists for that
	// panel.
	//

	m_labels = new char*[Nr*Nc];
	assert(m_labels);
	for( int i=0; i<Nr*Nc; i++ ) { 
		m_labels[i]=new char[256];
		assert( m_labels[i] );
		sprintf( m_labels[i], "" ); 
	}

	// Make sure that we know hereafter that this multiwin has been 
	// initialized!
	//
	m_initialized		= true;
}


// Here the caller gives an image and we will draw it into multiwindow.
// If the caller omits the parameter "r", then the argument "rc" is
// interpreted as the position in raster scan order.  If "r" is supplied,
// then rc is interpreted as the column, and r is the row.
// If 'rc' is not provided either, then the image is drawn to m_implicit_target,
// which is then incremented.  m_implicit_target is cleared on a draw
// command.
//
// TODO: it would be nice if we checked to make sure the user was
// drawing to a legal panel...
//
void CvMultiWin::paint( 
	IplImage* in, 
	int rc, 
	int r, 
	bool redraw_now 
) {

	if( ! m_initialized ) return;

	// First, figure out where we are supposed to draw the image 
	// we were given: 
	//
	int c;
	if( rc==-1 ) { rc = m_implicit_target++; }
	if( r!=-1 ) { 
		c = rc;
	} else {
		c = rc % m_columns;
		r = rc / m_columns;
	}

	// next compute the location of the panel into which we will draw:
	//
	int atx = M_SPC + c * (m_panel_width+M_SPC);
	int aty = M_SPC + r * (m_panel_height+M_SPC);

	// Now we have to create a new image which is the small form
	// of the input image image.  (Note that 'small' has the same
	// number of channels as 'in'.
	//
	IplImage* mini = cvCreateImage(
		cvSize( m_panel_width, m_panel_height ),
		in->depth,
		in->nChannels
	);
	cvResize( in, mini );

	IplImage* panel = cvCreateImage(
		cvSize( m_panel_width, m_panel_height ),
		m_image->depth,
		m_image->nChannels
	);

	if( mini->nChannels == 1 ) {
		cvCvtColor( mini, panel, CV_GRAY2BGR );
	} else {
		cvCopy( mini, panel );
	}
	cvReleaseImage( &mini );

	CvRect rect = cvRect( atx, aty, m_panel_width, m_panel_height );
	cvSetImageROI( m_image, rect );

//	cvFlip( panel, panel );
	cvPutText( panel, m_labels[r*m_columns+c], cvPoint(5,15), M_FONT, CV_RGB(0xff,0xff,0x00) );
	cvCopy( panel, m_image );
	cvResetImageROI( m_image );

	cvReleaseImage( &panel );

//		cvRectangle( m_image, cvPoint(atx+1,aty+1), cvPoint(m_panel_width,m_panel_height), CV_RGB(0,255,0), 1 );

	if( redraw_now ) {
		redraw();
	}


}
